@@ -381,21 +381,21 @@ add_proc (pid_t pid, pid_t ppid, long unsigned sjiffies,
 }
 
 /*------------------------------------------------------------------------*/
-/* read_proc now uses a similar method as procps for finding the process
- * name in the /proc filesystem. My thanks to Albert and procps authors.
+/* Uses a similar method as 'procps' for finding the process name in the
+ * /proc filesystem. My thanks to Albert and procps authors.
  */
 static void
 read_proc ()
 {
-  DIR *dir;
-  struct dirent *de;
-  FILE *file;
-  char *path, *buffer, *token;
-  pid_t pid, ppid;
-  int i, ch, tmp, size, pos, num_valid_results, empty;
+  int i, ch, tmp, size, pos, empty;
   long unsigned ujiffies, sjiffies;
+  char *path, *buffer, *token;
   unsigned long vsize;
+  struct dirent *de;
+  pid_t pid, ppid;
+  FILE *file;
   long rsize;
+  DIR *dir;
 
   buffer = malloc (size = 100);
 
@@ -417,7 +417,6 @@ SKIP:
       file = fopen (path, "r");
       free (path);
       if (!file) continue;
-      num_valid_results = 0;
       pos = 0;
       
       while ((ch = getc (file)) != EOF)
@@ -431,7 +430,6 @@ SKIP:
       fclose (file);
       
       ujiffies = sjiffies = -1;
-      num_valid_results = 0;
       vsize = 0;
       rsize = 0;
       
@@ -546,15 +544,6 @@ sample_recursive (double *time_ptr, double *mb_ptr)
 
 /*------------------------------------------------------------------------*/
 
-static void
-report (double time, double mb)
-{
-  fprintf (log, "[runlim] sample:\t\t%.1f seconds, %.1f MB\n", time, mb);
-  fflush (log);
-}
-
-/*------------------------------------------------------------------------*/
-
 struct itimerval timer, old_timer;
 static int caught_out_of_memory;
 static int caught_out_of_time;
@@ -564,17 +553,17 @@ static int caught_out_of_time;
 static void
 really_kill_child (void)
 {
-  usleep (10);
+  usleep (10000);
   kill (child_pid, SIGTERM);
-  usleep (10);
+  usleep (5000);
   kill (child_pid, SIGTERM);
-  usleep (10);
+  usleep (2000);
   kill (child_pid, SIGTERM);
-  usleep (100);
+  usleep (1000);
   kill (child_pid, SIGKILL);
-  usleep (10);
+  usleep (1000);
   kill (child_pid, SIGKILL);
-  usleep (10);
+  usleep (1000);
   kill (child_pid, SIGKILL);
 }
 
@@ -606,6 +595,18 @@ real_time (void)
 
 /*------------------------------------------------------------------------*/
 
+static void
+report (double time, double mb)
+{
+  double real = real_time ();
+  fprintf (log, 
+     "[runlim] sample:\t\t%.1f time, %.1f real, %.1f MB\n", 
+     time, real, mb);
+  fflush (log);
+}
+
+/*------------------------------------------------------------------------*/
+
 static void
 sampler (int s)
 {
@@ -834,8 +835,8 @@ main (int argc, char **argv)
 	hard_time_limit = (hard_time_limit * 101 + 99) / 100;	// + 1%
 	l.rlim_cur = l.rlim_max = hard_time_limit;
 	setrlimit (RLIMIT_CPU, &l);
-	l.rlim_cur = l.rlim_max = hard_time_limit << 20;
-	setrlimit (RLIMIT_RSS, &l);
+	l.rlim_cur = l.rlim_max = space_limit;
+	setrlimit (RLIMIT_AS, &l);
       }
       execvp (argv[i], argv + i);
       kill (getppid (), SIGUSR1);
@@ -862,33 +863,40 @@ main (int argc, char **argv)
     {
     case OK:
       fputs ("ok", log);
+      res = 0;
       break;
     case OUT_OF_TIME:
 FORCE_OUT_OF_TIME_ENTRY:
       fputs ("out of time", log);
-      res = 1;
+      res = 2;
       break;
     case OUT_OF_MEMORY:
       fputs ("out of memory", log);
-      res = 1;
+      res = 3;
       break;
     case SEGMENTATION_FAULT:
       fputs ("segmentation fault", log);
+      res = 4;
       break;
     case BUS_ERROR:
       fputs ("bus error", log);
+      res = 5;
       break;
     case FORK_FAILED:
       fputs ("fork failed", log);
+      res = 6;
       break;
     case INTERNAL_ERROR:
       fputs ("internal error", log);
+      res = 7;
       break;
     case EXEC_FAILED:
       fputs ("execvp failed", log);
+      res = 1;
       break;
     default:
       fprintf (log, "signal(%d)", s);
+      res = 11;
       break;
     }
   fputc ('\n', log);
